#!/usr/bin/perl -w

my $nodes2hostlist = "../nodes2hostlist.pl";
my $aquautil = "/home/tears/yipal/build/sfsnet-pastwatch.exp/aqua/aquautil";

my $pkfile = "";
my $skfile = "";
my $hostlist = "";
my $vfile = "";
my $logdir = "";
my $monitorlog = "";
my $nodelist = "";
my $distdir = "";
my $aquaconf = "";
my $aquaport = "";

my @hosts = ();
my %homedirs = ();
my %node_pks = ();

my $command;
my $configfile;


sub try_command {
    my ($backquote, $command) = @_;
    my $output = "";
    my $success = 0;

    print "Trying command: $command\n";
    
    eval {
	local $SIG{ALRM} = sub {die "alarm\n"};
	alarm 20;

	if ($backquote) {
	    $output = `$command`;
	    print $output;
	} else {
	    system($command);
	}
	$success = 1;
	alarm 0;
    };

    if (!$success) {
	eval {
	    local $SIG{ALRM} = sub {die "alarm\n"};
	    alarm 3;
	    system("echo \"Failed to execute command: $command\" | mail -s \"Aqua node_control failure\" yipal\@mit.edu");
	    alarm 0;
	};
    }
    return $output;
}

sub read_config {
    open(FILE, $configfile) or die "could not open [$configfile]\n";
    while(<FILE>) {
	if (/^(\S+)\s+(\S+)/) {
	    if ($1 eq "aqua_pk_file") {
		$pkfile = $2;
	    } elsif ($1 eq "aqua_sk_file") {
		$skfile = $2;
	    } elsif ($1 eq "hostlist_file") {
		$hostlist = $2;
	    } elsif ($1 eq "hostlist_version") {
		$vfile = $2;
	    } elsif ($1 eq "log_dir") {
		$logdir = $2;
	    } elsif ($1 eq "monitor_log") {
		$monitorlog = $2;
	    } elsif ($1 eq "nodes_file") {
		$nodelist = $2;
	    } elsif ($1 eq "dist_dir") {
		$distdir = $2;
	    } elsif ($1 eq "aqua_conf") {
		$aquaconf = $2;
	    } elsif ($1 eq "aqua_port") {
		$aquaport = $2;
	    } elsif ($1 eq "ssh_key") {
		$sshkey = $2;
	    } else {
		print "unknown parameter in config file: $_";
	    }
	} elsif (/^\s*$/) {
	} else {
	    print "malformed entry in config file: $_";
	}
    }
    close(FILE);

    # Read in node list
    open (FILE, $nodelist) or die "could not open [$nodelist]\n";
    while (<FILE>) {
	chomp;
	$_ =~ /^(\S+)\s+(\S+)\s+(\S+)\s*$/;
	push @hosts, $1;
	$homedirs{$1} = $2;
	$node_pks{$1} = $3;
    }
    close(FILE);
}

sub usage {
    print "usage:\n";
    print "  dht_control.pl startall live.conf\n";
    print "  dht_control.pl stopall  live.conf\n";
    print "  dht_control.pl syncall  live.conf\n";
    #print "  dht_control.pl startone live.conf hostname\n";
    #print "  dht_control.pl stopone  live.conf hostname\n";
    print "  dht_control.pl restartone live.conf hostname\n";

    print "  dht_control.pl regenhostlist live.conf\n";
    exit(1);
}

sub main {
    $command = shift @ARGV;
    $configfile = shift @ARGV;
    
    if ($command ne "startall" && 
	$command ne "stopall" && 
	$command ne "syncall" &&
	$command ne "restartone" &&
	$command ne "regenhostlist" &&
	$command ne "monitor"
	) {
	die "command must be startall, stopall or syncall\n";
    }

    &read_config();
    
    # Dispatch
    if ($command eq "startall" ||
	$command eq "stopall" ||
	$command eq "syncall") {
	&run_command_all();
    } elsif ($command eq "restartone") {
	$hostname = shift @ARGV;
	&stop_one($hostname);
	&start_one($hostname);
    } elsif ($command eq "regenhostlist") {
	&regen_hostlist();
    } elsif ($command eq "monitor") {
	&run_monitor();
    }
}


sub start_one {
    my ($host) = @_;
    my $homedir = $homedirs{$host};
    print "Starting node at $host\n";
    &try_command(0, "sleep 1; tcsh -c '(ssh -i $sshkey -x -a -1 $host \"$homedir/$distdir/startaqua.pl $homedir $distdir $aquaport $aquaconf $hostlist.signed >& $homedir/$distdir/aqua.log\") &' ");    
}

sub stop_one {
    my ($host) = @_;
    print "Stopping node at $host\n";
    #system ("sleep 1; tcsh -c '(ssh -i $sshkey -x -a -1 $host \"killall aquamulti\")' ");

    #system("ssh -x -a -1 $host echo \"`ps -xww | grep aqua-multi | grep -e '-p $aquaport' | cut -f 1 -d ' ' `\" | xargs kill ");
    sleep 1;
    my $processes = &try_command(1, "ssh -i $sshkey -x -a -1 $host ps -xww | grep aquamulti | grep -e \"-p $aquaport\" ");
    my @a = split /\n/, $processes;
    my $l;
    my @b = ();
    foreach $l (@a) {
	if ($l =~ /^\s*(\d+)\s+/) {
	    push @b, $1;
	}
    }
    $processes = join " ", @b;
    print "Killing processes $processes\n";
    &try_command(0, "ssh -i $sshkey -x -a -1 $host kill $processes");
}

sub sync_one {
    my ($host) = @_;
    my $homedir = $homedirs{$host};
    print "Syncing files to $host\n";
    system ("rsync -rz $distdir $host:$homedir");
}

sub regen_hostlist {
    open(FILE, $vfile) or die "Could not read version file: [$vfile]\n";
    my $version = -1;

    while (<FILE>) {
	if (/^(\d+)\s*$/) {
	    if ($1 >= 0) {
		$version = $1+1;
	    } else {
		print "Ignoring multiple versions specified in version file: [$vfile] $_";
	    }
	} else {
	    print "Ignoring malformed line in version file: [$vfile] $_";
	}
    }
    close(FILE);
    if ($version < 0) {	$version = 1; } 

    system("$nodes2hostlist $version $aquaport $nodelist > $hostlist");
    
    # Write out the version file with the new version number.
    open(OUT, ">$vfile") or die  "Could not write version file: [$vfile]\n";
    print OUT "$version\n";
    close(OUT);

    system("$aquautil -k $skfile -s $hostlist > $distdir/$hostlist.signed");
}

sub run_monitor {
    system("perl ../monitor.pl $aquaport \"../node_control.pl restartone $configfile\" $nodelist 2>&1 >> $monitorlog");
}

sub run_command_all {

    foreach $host (@hosts) {
	
	if ($command eq "startall") {
	    &start_one($host);
	} elsif ($command eq "stopall") {
	    &stop_one($host);
	} elsif ($command eq "syncall") {
	    &sync_one($host);
	}
    }
}

&main();

