/*
 *
 * Copyright (C) 2004 Alexander Yip (yipal@lcs.mit.edu)
 *
 *  Permission is hereby granted, free of charge, to any person obtaining
 *  a copy of this software and associated documentation files (the
 *  "Software"), to deal in the Software without restriction, including
 *  without limitation the rights to use, copy, modify, merge, publish,
 *  distribute, sublicense, and/or sell copies of the Software, and to
 *  permit persons to whom the Software is furnished to do so, subject to
 *  the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */
#include "gcc_compat.h"
#include <sys/types.h>
#include <sys/socket.h>
#include "sfsmisc.h"
#include "arpc.h"
#include "async.h"
#include "str.h"
#include "auditserver.h"
#include "audit_prot.h"
#include "merkle_tree.h"
#include "misc_utils.h"

auditserver::auditserver(ptr<axprt_stream> x, ptr<keyhash_idx> _kidx) {
  kidx = _kidx;
  clntsrv = asrv::alloc (x, audit_program_1,
	                 wrap (mkref (this), &auditserver::dispatch));
}
auditserver::~auditserver() {
}

void
auditserver::dispatch(svccb *sbp) {
  if (!sbp) {
    clntsrv = 0;
    return;
  } 

  auditstat res;
  switch(sbp->proc()) {

  case AUDITPROC_REMOVE_PK:
    // remove the block from keyhash_idx 
    {
      audit_removenotify_arg *arg = sbp->TEMPLATE getarg<audit_removenotify_arg> ();
      warn << "Server asked to remove " << armor64A(arg->removedblockid) << "\n";
      kidx->remove (arg->removedblockid);

      res = AUDIT_OK;
      sbp->reply(&res);
      return;
    }
  case AUDITPROC_REMOVE_CH:
    // remove the block from merkle tree
    {
      audit_removenotify_arg *arg = sbp->TEMPLATE getarg<audit_removenotify_arg> ();
      warn << "Server asked to remove " << armor64A(arg->removedblockid) << "\n";
      //block blk (to_merkle_hash(id2dbrec(arg->removedblockid)), NULL);
      //mtree->remove (&blk, false);

      res = AUDIT_OK;
      sbp->reply(&res);
      return;
    }
  default:
    warn << "Unknown RPC\n";
    res = AUDIT_ERR;
    sbp->reply(&res);
  }
}

void
auditserver::opensocket(str socketname, ptr<keyhash_idx> kidx) {
  int fd;

  unlink(socketname);
  fd = unixsocket(socketname);
  if (fd < 0) 
    fatal << "Error creating audit socket (UNIX) " << strerror(errno) << "\n";
  if (listen(fd, 5) < 0) {
    fatal("Error from listen: %m\n");
    close(fd);
  } else {
    fdcb (fd, selread, wrap(&auditserver::accept_audit_client, fd, kidx));
  }
}


void
auditserver::accept_audit_client(int fd, ptr<keyhash_idx> kidx) {
  sockaddr_un sun;
  bzero(&sun, sizeof(sun));
  socklen_t sunlen = sizeof(sun);
  int tfd = accept(fd, reinterpret_cast<sockaddr *> (&sun), &sunlen);
  if (tfd < 0) {
    warn << "could not accept unix socket\n";
    return;
  }

  ref<axprt_stream> x = axprt_stream::alloc (tfd, 1024*1025);

  // constructor of auditserver object calls mkref to maintain a
  // reference to itself until the program is gone.
  vNew refcounted<auditserver> (x, kidx);
}

