/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include <sys/time.h>
#include "stdinc.h"
#include "rxx.h"
#include "ns.h"
#include "cfsrw.h"
#include "misc_utils.h"
#include "mklog.h"

ptr<ns> nsobj = 0;
int ns_debug = 0;
int cfs_debug = 0;
static ptr<mklog> _mklog;

static void
usage ()
{
  warnx << "usage: " << progname << " [-S timestamp] [-s <keyfile>] [-c] proj\n";
  exit (1);
}

static void
create_done(cfsrw_status status) {
  exit (status);
}

void
start (bool create, bigint va, ptr<sfspriv> sk, str timestamp)
{
  _mklog = New refcounted<mklog> ();
  _mklog->create_log (sk, va, create, timestamp, true, wrap(create_done));
}

int
main (int argc, char **argv)
{
  char *sk_file = 0;
  bool create = false;
  str timestamp = "";

  setprogname (argv[0]);
  random_init ();

  argv++;
  argc--;

  if (argc < 1) 
    usage();
  
  while ( !strcmp(argv[0], "-s") || !strcmp(argv[0], "-c") || !strcmp(argv[0], "-S")) {

    if (!strcmp(argv[0], "-s")) {
      sk_file = argv[1];
      argv += 2;
      argc -= 2;
      if (argc == 0) 
	break;
    }
    
    if (!strcmp(argv[0], "-c")) {
      create = true;
      argv ++;
      argc --;
      if (argc == 0)
	break;
    }

    if (!strcmp(argv[0], "-S")) {
      timestamp = argv[1];
      argv += 2;
      argc -= 2;
      if (argc == 0)
	break;
    }
  }

  if ((argc != 1) || !sk_file)
    usage ();

  bigint va;
  char sym[1024];
  str dearm;
  rxx rx ("project:([0-9a-zA-Z+\\-]*)[/]*$");
  int r = readlink(argv[0], sym, 1024);
  if (r > 0) sym[r] = '\0';
  if (rx.search (argv[0]) || (r > 0 && rx.search (sym))) {
    dearm = dearmor64A(rx [1]);
  } else {
    dearm = dearmor64A(argv [0]);
  }
  if (dearm) 
    mpz_set_rawmag_be(&va, dearm.cstr(), dearm.len());
  else {
    warnx << "pastmklog: invalid project ID\n";
    exit(1);
  }

  ptr <sfspriv> sk;
  str key = file2wstr (sk_file);
  if (!key) {
    warn << sk_file << ": " << strerror (errno) << "\n";
    exit (1);
  }
  else if (!(sk = sfscrypt.alloc_priv (key, SFS_SIGN))) {
    warn << "could not decode " << sk_file << "\n";
    exit (1);
  }

  check_create_replicadir();
  nsobj = ns::alloc (va);
  nsobj->connect (wrap (start, create, va, sk, timestamp));
  amain();
}

