
// usage: chunk path
//
// chunk all files under path, create chunk statistics

#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <dirent.h>
#include <unistd.h>
#include "qhash.h"
#include "fingerprint.h"

template<>
struct hashfn<bigint> {
  hash_t operator() (const bigint &id) const { return id.getui (); }
};

struct bob {
  bigint key;
  unsigned n;
  static int cmp (const void *a_, const void *b_) {
    const bob *a = (bob *) a_, *b = (bob *) b_;
    return (int) (b->n - a->n);
  }
};

qhash<bigint,unsigned> bucket;
vec<bigint> keys;
str file;

void done ()
{
  bob b [keys.size ()];
  for (unsigned i=0; i<keys.size (); i++) {
    unsigned *n = bucket [keys [i]];
    if (n) {
      b [i].key = keys [i];
      b [i].n = *n;
    }
    else {
      assert (0);
    }
  }
  qsort (&b, keys.size (), sizeof (bob), &bob::cmp);
  
  strbuf out;
  out << "\n\n";
  out << "summary: total of " << keys.size () << " keys\n";
  for (unsigned i=0; i<keys.size (); i++) {
    out << b [i].key << ": " << b [i].n << "\n";
  }
  out.tosuio ()->output (1);
}

void
chunk_file (const char *path)
{
  int fd = open (path, O_RDONLY);
  unsigned char buf[8192];
  int count;
  Chunker chunker;

  while ((count = read (fd, buf, 8192))>0)
    chunker.chunk_data (buf, count);
  chunker.stop ();

  for (unsigned i=0; i<chunker.chunk_vector ().size (); i++) {
    chunk *c = chunker.chunk_vector ()[i];
    unsigned *n = bucket [c->hashn ()];
    if (n) {
      unsigned x = *n;
      x++;
      bucket.remove (c->hashn ());
      bucket.insert (c->hashn (), x);
    }
    else {
      bucket.insert (c->hashn (), 1);
      keys.push_back (c->hashn ());
    }

    strbuf out;
    out << c->hashn () << ":"
        // << " " << file << "@"
	<< c->pos ()
	<< "+" << c->count () << "\n";
    str x = c->buf ();
    if (x)
      out << x << "\n";
    out.tosuio ()->output (1);
  }
  close(fd);
}

void 
read_directory(const char *dpath)
{
  DIR *dirp;
  if ((dirp = opendir (dpath)) == 0) return;
  struct dirent *de = NULL;
  while ((de = readdir (dirp))) {
    if (strcmp (de->d_name, ".") == 0 || strcmp (de->d_name, "..") == 0)
      continue;
    char path[PATH_MAX];
    sprintf (path, "%s/%s", dpath, de->d_name);
    struct stat sb;
    lstat (path, &sb);
    if (S_ISDIR (sb.st_mode))
      read_directory (path);
    else if (S_ISREG (sb.st_mode) && !S_ISLNK(sb.st_mode)) {
      warn << "chunking " << path << "\n";
      file = str (path, strlen (path));
      chunk_file (path);
    }
  }
  closedir (dirp);
}

int 
main (int argc, char *argv[]) 
{
  if (argc != 2) {
    printf ("usage: %s path\n", argv[0]);
    return -1;
  }

  Chunker::avg_chunk_size = 256;
  Chunker::min_chunk_size = 128;
  Chunker::max_chunk_size = 1024;
  Chunker::keep_buf = 1;

  read_directory (argv[1]);
  done ();
}

