/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include <sys/types.h>
#include <dirent.h>
#include "rxx.h"
#include "qhash.h"
#include "btree.h"
#include "stdinc.h"
#include "opts.h"
#include "path.h"

extern ptr<cfsrw> _cfsrw;

str
translate_key_to_name (str t)
{
  str n;
  
  bigint x;
  if (!dearmor64A_bigint (t, x)) {
    warn << "cannot parse " << t << "\n";
    return n;
  }

  if (_cfsrw) {
    ptr<projinterface> proj = _cfsrw->getprojinf ();
    if (proj) {
      for (unsigned i=0; i<proj->nlogs (); i++)
        if (proj->get_log_ptr (i) == x)
	  return proj->get_log_label (i);
    }
  }

  return n;
}

str
translate_branch_tag (str t)
{
  // Argument <t> is in the form of an English branch tag.

  ptr<replica> r = _cfsrw->getreplica ();
  str n;
  vec<str> branch_heads;

  for (unsigned i=0; i<r->nbranches (); i++) {
    if (t == r->branch_tag (i)) {
      bigint c = r->branch_head_info (i).rptr;
      n = armor64A (c);
      branch_heads.push_back(n);
    }
  }

  if (branch_heads.size() == 1) 
    return branch_heads[0];
  
  if (arg_command != cmd_latest)
    warn << "Branch \"" << t << "\" has forked.  You must choose one head to track.\n";
  return NULL;
}

