
/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include <sys/types.h>
#include "sfsmisc.h"
#include "arpc.h"
#include "async.h"
#include "str.h"
#include "rxx.h"
#include "parseopt.h"
#include "refcnt.h"
#include "bigint.h"
#include "pastdbfe.h"
#include "pastupd.h"
#include "ns.h"
#include "penviron.h"

str home = pget_pwroot();
str rlist;

static void start_sync ();
extern void sync_repository (vec<str> n);

ptr<ns> nsobj = 0L;
bool check_mode = false;
bool sync_mode = false;
bool push_mode = false;

void
sync_resched ()
{
  if (check_mode)
    delaycb (SYNC_TIME, wrap (start_sync));
  else
    exit (0);
}

void
start_sync ()
{
  str buf = file2str (rlist);
  if (!buf) {
    warn << "no repository list, wait\n";
    sync_resched ();
    return;
  }
  vec<str> n;
  split (&n, "\n", buf);
  if (n.size () == 0) {
    sync_resched ();
    return;
  }
  sync_repository (n);
}

void
interrupted ()
{
  warn << "interrupted\n";
  if (nsobj)
    nsobj->exit_cleanup ();
  nsobj = 0;
  exit (1);
}

int
main (int argc, char **argv)
{
  setprogname (argv[0]);

  if (argc < 2) {
    warn << "must give a command\n";
    exit (1);
  }

  if (strncmp (argv [1], "check", 5) == 0)
    check_mode = true;
  else if (strncmp (argv [1], "sync", 4) == 0)
    sync_mode = true;
  else if (strncmp (argv [1], "push", 4) == 0)
    push_mode = true;
  else {
    warn << "Usage: " << argv [0] << " check|push|sync\n"
         << "   check: periodically tries to synchronize with DHT\n"
	 << "    sync: synchronize with DHT once\n"
	 << "    push: aggressively push all data onto DHT\n";
    exit (1);
  }

  pget_pwroot();  // Check that we can find pwroot
  if (!pgetenv ("PASTWATCH_TCPSOCKET"))
    fatal ("PASTWATCH_TCPSOCKET environment variable not defined");

  rlist = home << "/upd.repositories";
  str buf = file2str (rlist);
  if (!buf) {
    warn << "no repository list\n";
    exit (0);
  }
 
  sigcb (SIGHUP, wrap (interrupted));
  sigcb (SIGUSR1, wrap (interrupted));
  sigcb (SIGUSR2, wrap (interrupted));
  sigcb (SIGALRM, wrap (interrupted));
  sigcb (SIGINT, wrap (interrupted));
  sigcb (SIGTERM, wrap (interrupted));

  if (check_mode)
    delaycb (5, wrap (start_sync));
  else if (sync_mode || push_mode)
    delaycb (1, wrap (start_sync));
  
  amain ();
}

